@@ -15,7 +15,7 @@ from mdpo.mdpo2html import markdown_pofile_to_html
 
 
 DESCRIPTION = (
-    'HTML-produced-from-Markdown files translator using pofiles'
+    'HTML-produced-from-Markdown files translator using PO files'
     ' as reference.'
 )
 
@@ -36,7 +36,7 @@ def build_parser():
     )
     parser.add_argument(
         '-i', '--ignore', dest='ignore', default=[],
-        help='Filepaths to ignore when ``--pofiles`` argument value is a glob,'
+        help='Filepaths to ignore when \'--pofiles\' argument value is a glob,'
              ' as a list of comma separated values.',
         metavar='PATH_1,PATH_2...',
     )
